import { SchemaGraphology } from '@/lib/schema';
import { QueryElementTypes, QueryGraphNodes, QueryMultiGraph, SchemaEdge } from 'ts-common';

export const nodeEdges = (schemaNode: QueryGraphNodes, schemaGraph: SchemaGraphology): SchemaEdge[] => {
  if (!schemaNode.schemaKey) return [];
  if (schemaNode.type === QueryElementTypes.Entity) {
    // find entities on the edge of the current relation
    return schemaGraph.edges(schemaNode.schemaKey).map(edge => edge2schemaEdge(edge, schemaGraph));
  } else if (schemaNode.type === QueryElementTypes.Relation) {
    // find relations that are connected to the proper neighboring entity of current relation
    return relationEdgeIds(schemaNode.schemaKey, schemaGraph).map(edge => edge2schemaEdge(edge, schemaGraph));
  }
  return [];
};

export const relationEdgeIds = (schemaRelationKey: string, schemaGraph: SchemaGraphology): string[] => {
  const attributes = schemaGraph.getEdgeAttributes(schemaRelationKey);
  return [...schemaGraph.outboundEdges(attributes.to), ...schemaGraph.inboundEdges(attributes.from)];
};

export const edge2schemaEdge = (edgeKey: string, schemaGraph: SchemaGraphology): SchemaEdge => {
  const attrs = schemaGraph.getEdgeAttributes(edgeKey);
  return {
    ...attrs,
    name: attrs.name || edgeKey,
    to: attrs.to,
    from: attrs.from,
    collection: attrs.collection,
    attributes: attrs.attributes || [],
    label: edgeKey,
  };
};
